@@ -24,7 +24,7 @@ PYTEST_TO_TESTRAIL_STATUS = {
 DT_FORMAT = '%d-%m-%Y %H:%M:%S'
 
 TESTRAIL_PREFIX = 'testrail'
-
+TESTRAIL_DEFECTS_PREFIX = 'testrail_defects'
 ADD_RESULTS_URL = 'add_results_for_cases/{}'
 ADD_TESTRUN_URL = 'add_run/{}'
 CLOSE_TESTRUN_URL = 'close_run/{}'
@@ -48,6 +48,7 @@ class pytestrail(object):
     An alternative to using the testrail function as a decorator for test cases, since py.test may confuse it as a test
     function since it has the 'test' prefix
     '''
+
     @staticmethod
     def case(*ids):
         """
@@ -59,6 +60,17 @@ class pytestrail(object):
         """
         return pytest.mark.testrail(ids=ids)
 
+    @staticmethod
+    def defect(*defect_ids):
+        """
+                Decorator to mark defects with defect ids.
+
+                ie. @pytestrail.defect('PF-513', 'BR-3255')
+
+                :return pytest.mark:
+                """
+        return pytest.mark.testrail_defects(defect_ids=defect_ids)
+
 
 def testrail(*ids):
     """
@@ -100,6 +112,16 @@ def clean_test_ids(test_ids):
     return [int(re.search('(?P<test_id>[0-9]+$)', test_id).groupdict().get('test_id')) for test_id in test_ids]
 
 
+def clean_test_defects(defect_ids):
+    """
+        Clean pytest marker containing testrail defects ids.
+
+        :param list defect_ids: list of defect_ids.
+        :return list ints: contains list of defect_ids as ints.
+        """
+    return [(re.search('(?P<defect_id>.*)',defect_id).groupdict().get('defect_id')) for defect_id in defect_ids]
+
+
 def get_testrail_keys(items):
     """Return Tuple of Pytest nodes and TestRail ids from pytests markers"""
     testcaseids = []
@@ -119,7 +141,7 @@ def get_testrail_keys(items):
 class PyTestRailPlugin(object):
     def __init__(self, client, assign_user_id, project_id, suite_id, include_all, cert_check, tr_name, tr_description='', run_id=0,
                  plan_id=0, version='', close_on_complete=False, publish_blocked=True, skip_missing=False,
-                 milestone_id=None):
+                 milestone_id=None, custom_comment=None):
         self.assign_user_id = assign_user_id
         self.cert_check = cert_check
         self.client = client
@@ -136,6 +158,7 @@ class PyTestRailPlugin(object):
         self.publish_blocked = publish_blocked
         self.skip_missing = skip_missing
         self.milestone_id = milestone_id
+        self.custom_comment = custom_comment
 
     # pytest hooks
 
@@ -187,16 +210,34 @@ class PyTestRailPlugin(object):
         """ Collect result and associated testcases (TestRail) of an execution """
         outcome = yield
         rep = outcome.get_result()
+        defectids = None
+        if 'callspec' in dir(item):
+            test_parametrize = item.callspec.params
+        else:
+            test_parametrize = None
+        comment = rep.longrepr
+        if item.get_closest_marker(TESTRAIL_DEFECTS_PREFIX):
+            defectids = item.get_closest_marker(TESTRAIL_DEFECTS_PREFIX).kwargs.get('defect_ids')
         if item.get_closest_marker(TESTRAIL_PREFIX):
             testcaseids = item.get_closest_marker(TESTRAIL_PREFIX).kwargs.get('ids')
-
             if rep.when == 'call' and testcaseids:
-                self.add_result(
-                    clean_test_ids(testcaseids),
-                    get_test_outcome(outcome.get_result().outcome),
-                    comment=rep.longrepr,
-                    duration=rep.duration
-                )
+                if defectids != None:
+                    self.add_result(
+                        clean_test_ids(testcaseids),
+                        get_test_outcome(outcome.get_result().outcome),
+                        comment=comment,
+                        duration=rep.duration,
+                        defects=str(clean_test_defects(defectids)).replace('[', '').replace(']', '').replace("'", ''),
+                        test_parametrize = test_parametrize
+                    )
+                else:
+                    self.add_result(
+                        clean_test_ids(testcaseids),
+                        get_test_outcome(outcome.get_result().outcome),
+                        comment=comment,
+                        duration=rep.duration,
+                        test_parametrize=test_parametrize
+                    )
 
     def pytest_sessionfinish(self, session, exitstatus):
         """ Publish results in TestRail """
@@ -223,10 +264,12 @@ class PyTestRailPlugin(object):
 
     # plugin
 
-    def add_result(self, test_ids, status, comment='', duration=0):
+    def add_result(self, test_ids, status, comment='', defects=None, duration=0, test_parametrize=None):
         """
         Add a new result to results dict to be submitted at the end.
 
+        :param list test_parametrize: Add test parametrize to test result
+        :param defects: Add defects to test result
         :param list test_ids: list of test_ids.
         :param int status: status code of test (pass or fail).
         :param comment: None or a failure representation.
@@ -234,11 +277,13 @@ class PyTestRailPlugin(object):
         """
         for test_id in test_ids:
             data = {
-                'case_id': test_id,
-                'status_id': status,
-                'comment': comment,
-                'duration': duration
-            }
+                    'case_id': test_id,
+                    'status_id': status,
+                    'comment': comment,
+                    'duration': duration,
+                    'defects': defects,
+                    'test_parametrize': test_parametrize
+                }
             self.results.append(data)
 
     def add_results(self, testrun_id):
@@ -254,6 +299,7 @@ class PyTestRailPlugin(object):
         except NameError:
             converter = lambda s, c: str(bytes(s, "utf-8"), c)
         # Results are sorted by 'case_id' and by 'status_id' (worst result at the end)
+
         # Comment sort by status_id due to issue with pytest-rerun failures, for details refer to issue https://github.com/allankp/pytest-testrail/issues/100
         # self.results.sort(key=itemgetter('status_id'))
         self.results.sort(key=itemgetter('case_id'))
@@ -276,15 +322,29 @@ class PyTestRailPlugin(object):
         # Publish results
         data = {'results': []}
         for result in self.results:
-            entry = {'status_id': result['status_id'], 'case_id': result['case_id']}
+            entry = {'status_id': result['status_id'], 'case_id': result['case_id'], 'defects': result['defects']}
             if self.version:
                 entry['version'] = self.version
             comment = result.get('comment', '')
+            test_parametrize = result.get('test_parametrize', '')
+            entry['comment'] = u''
+            if test_parametrize:
+                entry['comment'] += u"# Test parametrize: #\n"
+                entry['comment'] += str(test_parametrize) + u'\n\n'
             if comment:
-                # Indent text to avoid string formatting by TestRail. Limit size of comment.
-                entry['comment'] = u"# Pytest result: #\n"
-                entry['comment'] += u'Log truncated\n...\n' if len(str(comment)) > COMMENT_SIZE_LIMIT else u''
-                entry['comment'] += u"    " + converter(str(comment), "utf-8")[-COMMENT_SIZE_LIMIT:].replace('\n', '\n    ')
+                if self.custom_comment:
+                    entry['comment'] += self.custom_comment + '\n'
+                    # Indent text to avoid string formatting by TestRail. Limit size of comment.
+                    entry['comment'] += u"# Pytest result: #\n"
+                    entry['comment'] += u'Log truncated\n...\n' if len(str(comment)) > COMMENT_SIZE_LIMIT else u''
+                    entry['comment'] += u"    " + converter(str(comment), "utf-8")[-COMMENT_SIZE_LIMIT:].replace('\n', '\n    ')
+                else:
+                    # Indent text to avoid string formatting by TestRail. Limit size of comment.
+                    entry['comment'] += u"# Pytest result: #\n"
+                    entry['comment'] += u'Log truncated\n...\n' if len(str(comment)) > COMMENT_SIZE_LIMIT else u''
+                    entry['comment'] += u"    " + converter(str(comment), "utf-8")[-COMMENT_SIZE_LIMIT:].replace('\n', '\n    ')
+            elif comment == '':
+                entry['comment'] = self.custom_comment
             duration = result.get('duration')
             if duration:
                 duration = 1 if (duration < 1) else int(round(duration))  # TestRail API doesn't manage milliseconds
@@ -328,9 +388,8 @@ class PyTestRailPlugin(object):
         else:
             self.testrun_id = response['id']
             print('[{}] New testrun created with name "{}" and ID={}'.format(TESTRAIL_PREFIX,
-                                                                              testrun_name,
-                                                                              self.testrun_id))
-
+                                                                             testrun_name,
+                                                                             self.testrun_id))
 
     def close_test_run(self, testrun_id):
         """
@@ -348,7 +407,6 @@ class PyTestRailPlugin(object):
         else:
             print('[{}] Test run with ID={} was closed'.format(TESTRAIL_PREFIX, self.testrun_id))
 
-
     def close_test_plan(self, testplan_id):
         """
         Closes testrun.
@@ -365,7 +423,6 @@ class PyTestRailPlugin(object):
         else:
             print('[{}] Test plan with ID={} was closed'.format(TESTRAIL_PREFIX, self.testplan_id))
 
-
     def is_testrun_available(self):
         """
         Ask if testrun is available in TestRail.
